package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"os"

	"focs.ji.sjtu.edu.cn/git/FOCS-dev/JOJ3/pkg/healthcheck"
)

// parseMultiValueFlag parses a multi-value command-line flag and appends its values to the provided slice.
// It registers a flag with the specified name and description, associating it with a multiStringValue receiver.
func parseMultiValueFlag(values *[]string, flagName, description string) {
	flag.Var((*multiStringValue)(values), flagName, description)
}

type multiStringValue []string

// Set appends a new value to the multiStringValue slice.
// It satisfies the flag.Value interface, allowing multiStringValue to be used as a flag value.
func (m *multiStringValue) Set(value string) error {
	*m = append(*m, value)
	return nil
}

func (m *multiStringValue) String() string {
	return fmt.Sprintf("%v", *m)
}

// Generally, err is used for runtime errors, and checkRes is used for the result of the checks.
func main() {
	var info []healthcheck.CheckStage
	var gitWhitelist, metaFile, releaseTags []string
	var tmp healthcheck.CheckStage

	rootDir := flag.String("root", "", "")
	repo := flag.String("repo", "", "")
	droneBranch := flag.String("droneBranch", "", "")
	releaseCategories := flag.String("releaseCategories", "", "")
	releaseNumber := flag.Int("releaseNumber", 0, "")
	// FIXME: for drone usage
	adminDir := flag.String("admin", "", "") // adminDir is for config files
	parseMultiValueFlag(&gitWhitelist, "whitelist", "")
	parseMultiValueFlag(&metaFile, "meta", "")
	parseMultiValueFlag(&releaseTags, "releaseTags", "")
	flag.Parse()

	tmp = healthcheck.RepoSize()
	info = append(info, tmp)

	tmp = healthcheck.ForbiddenCheck(*rootDir, gitWhitelist, *repo, *droneBranch)
	info = append(info, tmp)

	tmp = healthcheck.MetaCheck(*rootDir, metaFile)
	info = append(info, tmp)

	tmp = healthcheck.NonAsciiFiles(*rootDir)
	info = append(info, tmp)

	tmp = healthcheck.NonAsciiMsg(*rootDir)
	info = append(info, tmp)

	// TODO: find a way to test the release tag
	tmp = healthcheck.CheckReleases(*rootDir, *releaseCategories, *releaseNumber)
	info = append(info, tmp)

	// FIXME: for drone usage
	tmp = healthcheck.Verify(*rootDir, *adminDir)
	info = append(info, tmp)

	jsonData, err := json.Marshal(info)
	if err != nil {
		fmt.Fprint(os.Stderr, err)
		os.Exit(1)
	}
	fmt.Printf("%s", jsonData)
}
